/** 
 * @fileoverview 
 *
 * @author Jonny Morrill jonny@morrill.me
 * @version 0.1
 */

(function() {
    var lastTime = 0;
    var vendors = ['ms', 'moz', 'webkit', 'o'];
    for(var x = 0; x < vendors.length && !window.requestAnimationFrame; ++x) {
        window.requestAnimationFrame = window[vendors[x]+'RequestAnimationFrame'];
        window.cancelAnimationFrame = 
          window[vendors[x]+'CancelAnimationFrame'] || window[vendors[x]+'CancelRequestAnimationFrame'];
    }
 
    if (!window.requestAnimationFrame)
        window.requestAnimationFrame = function(callback, element) {
            var currTime = new Date().getTime();
            var timeToCall = Math.max(0, 16 - (currTime - lastTime));
            var id = window.setTimeout(function() { callback(currTime + timeToCall); }, 
              timeToCall);
            lastTime = currTime + timeToCall;
            return id;
        };
 
    if (!window.cancelAnimationFrame)
        window.cancelAnimationFrame = function(id) {
            clearTimeout(id);
        };
}());

(function() {
	
	Event.observe(window, 'load', main);
	
	var WIDTH, HEIGHT;
	
	var DURATION = 2000;
	var FREQUENCY = 40;
	var AMPLITUDE = 32;
	
	var tempCanvasArr = [];
	var tempContextArr = [];
	
	var canvasArr = [];
	var contextArr = [];
	
	var canvas,
		context,
		image;
	
	var xShake = null;
	var yShake = null;
	
	var startTime = null;
	
    function main()
    {
		
		image = $('image');
		
		tempCanvasArr.push(new Element('canvas'));
		tempCanvasArr.push(new Element('canvas'));
		
		canvasArr.push($('canvas_a'));
		canvasArr.push($('canvas_b'));
		
		WIDTH = canvasArr[0].width;
		HEIGHT = canvasArr[0].height;
		
		tempCanvasArr[0].width = WIDTH;
		tempCanvasArr[0].height = HEIGHT;
		tempCanvasArr[1].width = WIDTH;
		tempCanvasArr[1].height = HEIGHT;
		
		canvas = $('canvas_v');
		context = canvas.getContext('2d');
		
		Event.observe('shakeBtn', 'click', handleStartClick);
		
		tempContextArr.push(tempCanvasArr[0].getContext('2d'));
		tempContextArr.push(tempCanvasArr[1].getContext('2d'));
		
		contextArr.push(canvasArr[0].getContext('2d'));
		contextArr.push(canvasArr[1].getContext('2d'));
		
		handleStartClick();
    }
	
	function handleStartClick()
	{
		
		DURATION = parseFloat($('durationTxt').value) * 1000;
		FREQUENCY = parseInt($('frequencyTxt').value);
		AMPLITUDE = parseInt($('amplitudeTxt').value);
		
		canvasArr[0].width = WIDTH;
		canvasArr[1].width = WIDTH;
		
		tempCanvasArr[0].width = WIDTH;
		tempCanvasArr[1].width = WIDTH;
		
		xShake = new Shake(DURATION, FREQUENCY);
		yShake = new Shake(DURATION, FREQUENCY);
		
		var t, x, y, s;
		
		for(s = 0; s <= xShake.samples.length; s++) {
			t = s / xShake.frequency * 1000;
			x = s / xShake.samples.length * WIDTH;
			y = (xShake.amplitude(t)*HEIGHT + HEIGHT) / 2;
			tempContextArr[0].lineTo(x, y);
		}
		
		for(s = 0; s <= yShake.samples.length; s++) {
			t = s / yShake.frequency * 1000;
			x = s / yShake.samples.length * WIDTH;
			y = (yShake.amplitude(t)*HEIGHT + HEIGHT) / 2;
			tempContextArr[1].lineTo(x, y);
		}
		
		tempContextArr[0].stroke();
		tempContextArr[1].stroke();
		
		xShake.start();
		yShake.start();
		
		startTime = new Date().getTime();
		
		window.requestAnimationFrame(update);
		
	}
	
	function update(t)
	{
		xShake.update();
		yShake.update();
		
		if(xShake.isShaking || yShake.isShaking) {
			window.requestAnimationFrame(update);
		}
		
		var ctx = context;
		canvas.width = canvas.width;
		
		var x = xShake.amplitude() * AMPLITUDE;
		var y = yShake.amplitude() * AMPLITUDE;
		ctx.drawImage(image, 
			x - (800 - canvas.width)/2,
			y - (600 - canvas.height)/2
		)
		
		x = (t - startTime) / DURATION * WIDTH;
		
		ctx = contextArr[0];
		canvasArr[0].width = WIDTH;
		ctx.drawImage(tempCanvasArr[0], 0, 0);
		ctx.moveTo(x, 0);
		ctx.lineTo(x, HEIGHT);
		ctx.stroke();
		
		ctx = contextArr[1];
		canvasArr[1].width = WIDTH;
		ctx.drawImage(tempCanvasArr[1], 0, 0);
		ctx.moveTo(x, 0);
		ctx.lineTo(x, HEIGHT);
		ctx.stroke();
		
	}
	
	/**
	 * @class Initializes a 1D shakeing pattern
	 * @param {int} duration The length of the shake in miliseconds
	 * @param {int} frequency The frequency of the shake in Hertz
	 */
	var Shake = function(duration, frequency)
	{
		// The duration in miliseconds
		this.duration = parseInt(duration);
		
		// The frequency in Hz
		this.frequency = parseInt(frequency);
		
		// The sample count = number of peaks/valleys in the Shake
		var sampleCount = (duration/1000) * frequency;
		
		// Populate the samples array with randomized values between -1.0 and 1.0
		this.samples = [];
		for(var i = 0; i < sampleCount; i++) {
			this.samples.push(Math.random() * 2 - 1);
		}
		
		// Init the time variables
		this.startTime = null;
		this.t = null
		
		// Flag that represents if the shake is active
		this.isShaking = false;
	};
	
	/**
	 * Start the shake, initializing time variables and flags
	 */
	Shake.prototype.start = function()
	{
		this.startTime = new Date().getTime();
		this.t = 0;
		this.isShaking = true;
	};
	
	/**
	 * Update the shake, setting the current time variable
	 */
	Shake.prototype.update = function()
	{
		this.t = new Date().getTime() - this.startTime;
		if(this.t > this.duration) this.isShaking = false;
	};
	
	/**
	 * Retrieve the amplitude. If "t" is passed, it will get the amplitude for the
	 * specified time, otherwise it will use the internal time.
	 * @param {int} t (optional) The time since the start of the shake in miliseconds
	 */
	Shake.prototype.amplitude = function(t)
	{
		// Check if optional param was passed
		if(t == undefined) {
			// return zero if we are done shaking
			if(!this.isShaking) return 0;
			t = this.t;
		}
		
		// Get the previous and next sample
		var s = t / 1000 * this.frequency;
		var s0 = Math.floor(s);
		var s1 = s0 + 1;
		
		// Get the current decay
		var k = this.decay(t);
		
		// Return the current amplitide 
		return (this.noise(s0) + (s - s0)*(this.noise(s1) - this.noise(s0))) * k;
	};
	
	/**
	 * Retrieve the noise at the specified sample.
	 * @param {int} s The randomized sample we are interested in.
	 */
	Shake.prototype.noise = function(s)
	{
		// Retrieve the randomized value from the samples
		if(s >= this.samples.length) return 0;
		return this.samples[s];
	};
	
	/**
	 * Get the decay of the shake as a floating point value from 0.0 to 1.0
	 * @param {int} t The time since the start of the shake in miliseconds
	 */
	Shake.prototype.decay = function(t)
	{
		// Linear decay
		if(t >= this.duration) return 0;
		return (this.duration - t) / this.duration;
	};
	
	
}());